function Character(initBlocks, h,s,l){
	this.x=util.randRange(1000,999999999);
	this.y=util.randRange(1000,999999999);

	this.maxSkipCounter = 10;
	this.skipCounter = util.randRange(1,this.maxSkipCounter);

	this.h=h;
	this.s=s;
	this.l=l;

	this.invisibility = 0;
	this.speedPlusPlus = 0;

	this.animationOffset = {
		x:0,
		y:0
	}

	this.tile = null;

	this.visible = false;

	if(initBlocks==undefined || initBlocks.length==0){
		this.blocks = [
			new Block(0,0, this)
		];
		//make a third of monsters small
		var rand = Math.random();


		if(rand<.20){
			var growRate = .5;
			var decayRate = 0;
		}else if(rand<.40){
			var growRate = 0;
			var decayRate = 0;
		}else if(rand<.90){
			var growRate = .1*game.effectiveLevel;
			var decayRate = .1*game.effectiveLevel;
		}else{
			var growRate = .2+.1*game.effectiveLevel;
			var decayRate = .2+.1*game.effectiveLevel;
		}

		this.grow(this.blocks[0],growRate,decayRate);
	}else{
		this.blocks=[];
		for(var i=0;i<initBlocks.length;i++){
			this.blocks.push(new Block(
				initBlocks[i].x,
				initBlocks[i].y,
				this
			));
		}
	}

	

	this.canvas = null;
}

//only for AI
Character.prototype.performActions = function(){
	this.skipCounter++;
	if(this.skipCounter % this.maxSkipCounter == 0){
		//do nothing 10% of the time, so we don't screw the player
		return;
	}

	var ableToFollow = false;
	if(game.state=="running" && pc.invisibility<=0 && util.distance(this.x,this.y,pc.x,pc.y)<game.viewRadius){
		var path = this.pathfind(pc.x,pc.y);

		if(path != null && this.fits(path.x,path.y,pc)){
			ableToFollow = true;
			this.tryMove(
				path.x-this.x,
				path.y-this.y
			);
		}
	}
	if(!ableToFollow){
		var dir = util.randomDirection();
		if(this.fits(this.x+dir.x,this.y+dir.y)){
			this.tryMove(dir.x,dir.y);
		}
	}
}

Character.prototype.findBlock = function(x,y){
	for(var i=0;i<this.blocks.length;i++){
		if(this.blocks[i].x==x && this.blocks[i].y==y){
			return this.blocks[i];
		}
	}
	return null;
}

Character.prototype.cancer = function(){
	if(this.cancerBlocks==undefined){
		this.cancerBlocks = this.blocks.slice();
	}

	util.shuffle(this.cancerBlocks);
	var directions = util.getDirections();

	for(var i=0;i<1000;i++){
		if(this.cancerBlocks.length==0){
			return;
		}
		var block = this.cancerBlocks.shift();
		if(Math.abs(block.x-this.blocks[0].x)>game.viewRadius+1 ||
			Math.abs(block.y-this.blocks[0].y)>game.viewRadius+1){
			continue;
		}
		for(var j=0;j<directions.length;j++){
			var neighbor = this.findBlock(
				block.x+directions[j].x,
				block.y+directions[j].y
			);
			if(neighbor==null){
				var newBlock = new Block(
					block.x+directions[j].x,
					block.y+directions[j].y,
					this
				);
				this.blocks.push(newBlock);
				this.cancerBlocks.push(newBlock);
				this.cancerBlocks.push(block);
				return;
			}
		}
	}
}

Character.prototype.grow = function(block,growChance, decay){
	if(this.blocks.length>300){
		return;
	}

	if(!block.visited){
		block.visited = true;
		var directions = util.getDirections();
		for(var i=0;i<directions.length;i++){
			if(Math.random()<=growChance){
				var neighbor = this.findBlock(
					block.x+directions[i].x,
					block.y+directions[i].y
				);
				if(neighbor==null){
					var newBlock = new Block(
						block.x+directions[i].x,
						block.y+directions[i].y,
						this
					);
					this.blocks.push(newBlock);

					this.grow(newBlock, growChance*decay, decay);
				}
			}
		}
	}
}

Character.prototype.visitBlock = function(block){
	if(!block.visited){
		block.visited = true;
		var directions = util.getDirections();
		for(var i=0;i<directions.length;i++){
			var neighbor = this.findBlock(
				block.x+directions[i].x,
				block.y+directions[i].y
			);

			if(neighbor != null){
				this.visitBlock(neighbor);
			}
		}
	}
}

Character.prototype.deleteOrphanBlocks = function(){
	for(var i=0;i<this.blocks.length;i++){
		this.blocks[i].visited = false;
	}

	var center = this.findBlock(0,0);
	this.visitBlock(center);
	
	for(var i=this.blocks.length-1;i>=0;i--){
		if(!this.blocks[i].visited){
			this.blocks[i].deleteYoself();
		}
	}
}

Character.prototype.getViewableTiles = function(){
	var tiles = [];
	var r = Math.ceil(game.viewRadius+1);
	for(var x=this.x-r;x<=this.x+r;x++){
		for(var y=this.y-r;y<=this.y+r;y++){
			tiles.push(map.getTile(x,y));
		}
	}
	return tiles;
}

Character.prototype.pathfind = function(){
	var path = [];

	var sourceTile = this.tile;
    var destTile =  pc.tile;
	var manhattan = Math.abs(this.x-pc.x)+Math.abs(this.y-pc.y);
	
	if(util.distance(this.x,this.y,pc.x,pc.y)>game.viewRadius+1) return null;
	
	if(this.x==pc.x && this.y==pc.y) return;
    
    var displayTiles = this.getViewableTiles();
    for(var i=0;i<displayTiles.length;i++){
        displayTiles[i].astar={
        	visited:false,
        	f:10000
        };
    }
    
    candidates = [sourceTile];

    sourceTile.astar.g=0;
    sourceTile.astar.f=manhattan;

    var timeout= 2000;
    while(candidates.length>0){
        timeout--;
        if(timeout<=0)break;
        
        candidates.sort(function(a,b){
           return a.astar.f - b.astar.f;
        });

        var c = candidates[0];

        //check for goal

        //todo: change "goal" to be touching a player tile
        for(var i=0;i<this.blocks.length;i++){
        	var testTile = map.getTile(
        		c.x+this.blocks[i].x,
        		c.y+this.blocks[i].y
        	);
        	if(testTile.block!=null && testTile.block.parentCharacter.equals(pc)){

            	var p = this.path(c,sourceTile);
            	return p;
        	}
        }

        candidates.shift();
        
        c.astar.visited=true;
        
        var neighbors = c.getAdjacentNeighbors(true);
        
        for(var i=0;i<neighbors.length;i++){
        	var n = neighbors[i];

        	if(util.isInArray(displayTiles,n)){
				if(n.astar.visited) continue;

				var tentative = c.astar.g+1;
				
				
				if(!this.fits(n.x,n.y,pc)){
					continue;
				}
				
				var found =util.isInArray(candidates,n);
				if(!found || n.astar.g > tentative){
					n.astar.from = c;
					n.astar.g=tentative;
					n.astar.f=n.astar.g+Math.abs(n.x-pc.x)+Math.abs(n.x-pc.y);
					
					if(!found && (!n.blocking || n.equals(destTile))){
						if(util.distance(n.x,n.y,pc.x,pc.y)<game.viewRadius+1)
						candidates.push(n);
					}
				}
        	}
        }
    }
    return null;
}
                                             
Character.prototype.path = function(tile, start){
    var from = tile.astar.from; 
    if(from.equals(start)){
         return tile;   
    }else{
         return this.path(from,start);
    }
}



Character.prototype.equals = function(character){
	if(this.x==character.x && this.y==character.y){
		return true;
	}else{
		return false;
	}
}

//return false if you are unable to place the monster
Character.prototype.place = function(extraSpace, avoidPlayer){
	for(var i=0;i<1000;i++){
		//check to see if we fit
		//todo: more complicated fit logic

		var tile = map.randomTile();

		var nearbyPlayer = false;
		if(avoidPlayer){
			var r = Math.ceil(game.viewRadius);
			for(var x=tile.x-r;x<=tile.x+r;x++){
				for(var y=tile.y-r;y<=tile.y+r;y++){
					var testTile = map.getTile(x,y);
					if(testTile.block && testTile.equals(pc.tile)){
						nearbyPlayer = true;
						break;
					}
				}
				if(nearbyPlayer)break;
			}
			if(nearbyPlayer)continue;
		}

		if(this.fits(tile.x, tile.y, undefined, true) &&
		(extraSpace==undefined ||
		(this.fits(tile.x+1,tile.y)
		&& this.fits(tile.x-1,tile.y)
		&& this.fits(tile.x,tile.y+1)
		&& this.fits(tile.x,tile.y-1)))){
			this.moveToTile(tile);
			return true;
		}
	}
	

	return false;
}

Character.prototype.moveToTile = function(tile){
	var patternBlockHit = false;

	for(var i=0;i<this.blocks.length;i++){
		var oldTile = map.getTile(
			this.blocks[i].x+this.x,
			this.blocks[i].y+this.y
		);
		oldTile.block = null;
	}
	for(var i=0;i<this.blocks.length;i++){
		var newTile = map.getTile(
			this.blocks[i].x+tile.x,
			this.blocks[i].y+tile.y
		);

		newTile.block = this.blocks[i];

		if(newTile.item!=null){
			newTile.item.pickup(this.blocks[i],newTile);
		}

		if(newTile.patternBlock){
			patternBlockHit = true;
		}
	}

	//turn off invisibility if next to a monster
	if(this.invisibility>0){
		if(this.getOverlaps(tile.x+1,tile.y).monsterOverlaps.length>0
		|| this.getOverlaps(tile.x-1,tile.y).monsterOverlaps.length>0
		|| this.getOverlaps(tile.x,tile.y+1).monsterOverlaps.length>0
		|| this.getOverlaps(tile.x,tile.y-1).monsterOverlaps.length>0){
			this.invisibility = 0;
		}
	}


	//ACTUALLY MOVE
	this.x = tile.x;
	this.y = tile.y;
	this.tile = tile;



	if(this.equals(pc)){
		if(game.level >= 7 && tile.equals(game.gateTile)){
			setTimeout(function(){
				game.end();
			},700);
			return;
		}

		if(tile.equals(game.crownTile)){
			game.completeLevel();
		}

		game.prayer=0;

		//a tile was hit by our block
		game.patternBlockHit = patternBlockHit;

		//a tile was hit by our center block (we might have it)
		if(tile.patternBlock){
			tile.patternBlock.parentPattern.checkMatch();
		}
	}
}

//called on game loop (e.g. animation), not moves
Character.prototype.update = function(dt){
	if(this.speedPlusPlus>0){
		this.animationOffset.x *= .5;
		this.animationOffset.y *= .5;
	}else{
		this.animationOffset.x *= .85;
		this.animationOffset.y *= .85;
	}

	if(this.animationOffset.x <1 && this.animationOffset.x >-1){
		this.animationOffset.x = 0;
	}

	if(this.animationOffset.y <1 && this.animationOffset.y >-1){
		this.animationOffset.y = 0;
	}
}

//saves a canvas for drawing and also calculates corners
Character.prototype.calculateCanvas = function(skipItemDraw, atAlpha){
	if(atAlpha==undefined){
		atAlpha=1;
	}
	var tileSize = map.tileSize;
	var canvas = document.createElement('canvas');

	//calculate total size
	//calculate left corner offset

	this.minX=10000;
	this.minY=10000;
	this.maxX=-10000;
	this.maxY=-10000;

	for(var i=0;i<this.blocks.length;i++){
		this.minX = Math.min(this.blocks[i].x, this.minX);
		this.maxX = Math.max(this.blocks[i].x, this.maxX);
		this.minY = Math.min(this.blocks[i].y, this.minY);
		this.maxY = Math.max(this.blocks[i].y, this.maxY);
	}

	var width = this.maxX-this.minX+1;
	var height = this.maxY-this.minY+1;

    canvas.width = tileSize * width+2*map.lineSize;
    canvas.height = tileSize * height+2*map.lineSize;




    var ctx = canvas.getContext('2d');





    //OUTLINES
    if(game.targetZoom>=1){
		ctx.fillStyle = 'black';
		for(var i=0;i<this.blocks.length;i++){
			/*if(map.getTile(
				this.blocks[i].x+this.x,
				this.blocks[i].y+this.y
			).explored){*/
				ctx.fillRect(
					(this.blocks[i].x-this.minX)*tileSize,
					(this.blocks[i].y-this.minY)*tileSize,
					tileSize+2*map.lineSize,
					tileSize+2*map.lineSize
				);
			//}
		}
	}

			

	
	for(var i=0;i<this.blocks.length;i++){
		/*if(map.getTile(
			this.blocks[i].x+this.x,
			this.blocks[i].y+this.y
		).explored){*/
			ctx.fillStyle = this.blocks[i].color.string;
				
			ctx.fillRect(
				(this.blocks[i].x-this.minX)*tileSize+map.lineSize,
				(this.blocks[i].y-this.minY)*tileSize+map.lineSize,
				tileSize,
				tileSize
			);

			if(this.blocks[i].item!=null && skipItemDraw == undefined){
				this.blocks[i].item.draw(
					ctx,
					true,
					(this.blocks[i].x-this.minX+.5)*map.tileSize+map.lineSize,
					(this.blocks[i].y-this.minY+.5)*map.tileSize+map.lineSize,
	  				Math.ceil(map.tileSize*.2)
				);
			}
		//}
		
	}

	//todo: maybe make the @ "filled"?
	if(game.targetZoom>=1){
		var atPoints = [
			[.5,.5],
			[.6,.5],
			[.6,.4],
			[.4,.4],
			[.4,.6],
			[.7,.6],
			[.7,.3],
			[.3,.3],
			[.3,.7],
			[.7,.7]
		];


		ctx.strokeStyle = 'black';
		ctx.beginPath();
		ctx.moveTo(
			Math.floor(tileSize*atPoints[0][0] - this.minX*tileSize),
			Math.floor(tileSize*atPoints[0][1] - this.minY*tileSize)
		);

		for(var i=1;i<atPoints.length-1;i++){
			ctx.lineTo(
				Math.floor(tileSize*atPoints[i][0] - this.minX*tileSize),
				Math.floor(tileSize*atPoints[i][1] - this.minY*tileSize)
			);
		}
		ctx.lineWidth = Math.ceil(tileSize/12000)*2;
		ctx.lineTo(
			Math.floor(tileSize*atPoints[atPoints.length-1][0]+ctx.lineWidth/2 - this.minX*tileSize),
			Math.floor(tileSize*atPoints[atPoints.length-1][1] - this.minY*tileSize)
		);
		ctx.globalAlpha = atAlpha;
	    ctx.globalCompositeOperation = "destination-out";
	    ctx.stroke();
		ctx.globalCompositeOperation = "source-over";
		ctx.globalAlpha = 1;
	}

	
		

	this.canvas = canvas;
}

Character.prototype.draw = function(canvasCtx){
	if(this.canvas==null){
		return;
	}

	//this.calculateCanvas();

	if(this.speedPlusPlus>0){
		//bounce only when you have a free move
		if(game.tickCount%2==1){
			var bounce = 0;
		}else{
			var bounce = (map.tileSize/3)
		}
		this.singleDraw(canvasCtx, 0, -bounce*Math.abs(Math.sin(game.speedCounter/4)));
	}else{	
		if(this.invisibility>0){
			canvasCtx.globalAlpha = .4;
		}
		this.singleDraw(canvasCtx,0,0);
	}

	canvasCtx.globalAlpha = 1;
}

Character.prototype.singleDraw = function(canvasCtx,shakeX,shakeY){
	var camera = game.getCamera();
	var startX = (this.animationOffset.x + (this.minX+this.x)*map.tileSize)*camera.zoom + camera.xOffset - map.lineSize;
	var startY = (this.animationOffset.y + (this.minY+this.y)*map.tileSize)*camera.zoom + camera.yOffset - map.lineSize;

	startX+=shakeX*camera.zoom;
	startY+=shakeY*camera.zoom;

	canvasCtx.drawImage(this.canvas, startX, startY, this.canvas.width*camera.zoom, this.canvas.height*camera.zoom);

	canvasCtx.globalAlpha = 1;
}

Character.prototype.fits = function(x,y, ignoreCharacter, checkGate){
	var tile = map.getTile(x,y);
	if(checkGate && tile.equals(game.crownTile)){
		return false;
	}

	for(var i=0;i<this.blocks.length;i++){
		var tile = map.getTile(
			x+this.blocks[i].x,
			y+this.blocks[i].y
		);
		if(tile.blocking){
			return false;
		}

		if(tile.block &&
		!this.equals(tile.block.parentCharacter)
		&& (ignoreCharacter == undefined || !ignoreCharacter.equals(tile.block.parentCharacter))){
			return false;
		}

		if(tile.diamond){
			return false;
		}
	}
	return true;
}

//returns arrays of monster and wall overlaps of a theoretical position
Character.prototype.getOverlaps = function(x,y, ignoreCharacter){
	var monsterOverlaps = [];
	var wallOverlaps = [];
	var diamondOverlaps = [];
	for(var i=0;i<this.blocks.length;i++){
		var tile = map.getTile(
			x+this.blocks[i].x,
			y+this.blocks[i].y
		);
		if(tile.blocking){
			wallOverlaps.push(this.blocks[i]);
		}

		if(tile.block &&
		!this.equals(tile.block.parentCharacter)
		&& (ignoreCharacter == undefined || !ignoreCharacter.equals(tile.block.parentCharacter))){
			monsterOverlaps.push(this.blocks[i]);
		}

		if(tile.diamond){
			diamondOverlaps.push(this.blocks[i]);
		}
	}

	return {
		monsterOverlaps: monsterOverlaps,
		wallOverlaps: wallOverlaps,
		diamondOverlaps: diamondOverlaps
	}
}

//return true if the movement workeed
Character.prototype.tryMove = function(dx, dy){
	var overlaps = this.getOverlaps(this.x+dx,this.y+dy);


	var newX = this.x+dx;
	var newY = this.y+dy;
	var newTile = map.getTile(newX, newY);

	if(overlaps.diamondOverlaps.length>0 && overlaps.monsterOverlaps.length==0 && overlaps.wallOverlaps.length==0){
		var diamonds = overlaps.diamondOverlaps;
		for(var i=0;i<diamonds.length;i++){
			var block = diamonds[i];

			if(block.isCenter()){
				if(this.equals(pc)){
					game.state="dead";
				}

				game.deleteMonster(this);
				break;
			}

			block.deleteYoself();
		}
		if(this.blocks.length>0){
			this.deleteOrphanBlocks();
		}

		this.moveToTile(newTile);
		this.animationOffset.x += -dx * map.tileSize;
		this.animationOffset.y += -dy * map.tileSize;

		return true;
	}else if(overlaps.wallOverlaps.length>0){
		this.animationOffset.x += dx * map.tileSize*.5;
		this.animationOffset.y += dy * map.tileSize*.5;

		var usedItem = false;

		for(var i=0;i<overlaps.wallOverlaps.length;i++){
			var wallBlock = overlaps.wallOverlaps[i];
			if(wallBlock.item!=null){
				wallBlock.item.use(dx,dy);
				usedItem = true;
			}
		}

		if(usedItem){
			return true;
		}else{
			if(this.equals(pc)){
				if(game.direction != dx+","+dy){
					game.direction = dx+","+dy;
					game.prayer=1;
				}else{
					game.prayer++;
					if(game.prayer>=7 && this.blocks.length>1){
						game.prayer = 0;
						var numWhite = 0;
						for(var i=this.blocks.length-1;i>=0;i--){
							if(!this.blocks[i].isCenter()){
								var deadBlockTile = this.blocks[i].getTile();
								var centerDistance = util.distance(this.blocks[i].x,this.blocks[i].y,0,0);

								this.blocks[i].deleteYoself();
								var dangerChance = game.prayerDanger*centerDistance/5.0;
								if(centerDistance>1 && (Math.random() < dangerChance)){
									var monster = new Character([{x:0,y:0}],0, 0, 100);
									game.monsters.push(monster);
									monster.moveToTile(deadBlockTile);
									monster.calculateCanvas();
									numWhite++;
								}
							}
						}

						game.prayerDanger += (1-game.prayerDanger)*.1;

						this.calculateCanvas();
					}
				}
			}
			return false;
		}
	}else{
		if(overlaps.monsterOverlaps.length>0){
			var blocks = overlaps.monsterOverlaps;


			//this is all kludgy because it's possible that multiple enemies were overlapped at once 

			//use 'x,y' as ID
			overlappedEnemies = [];

			//contains all overlap blocks but split by parents
			splitBlocks = {};

			//absorption means we get bigger; it also means we don't move
			var absorption = false;

			for(var i=0;i<blocks.length;i++){
				var parentId = map.getTile(
					newX + blocks[i].x,
					newY + blocks[i].y
				).block.getParentId();

				if(overlappedEnemies.indexOf(parentId)==-1){
					overlappedEnemies.push(parentId);
					splitBlocks[parentId] = [];
				}
				splitBlocks[parentId].push(blocks[i]);
			}
			
			var ourSize = this.blocks.length;



			for(var e=0;e<overlappedEnemies.length;e++){
				var enemyId = overlappedEnemies[e];

				var firstBlock = splitBlocks[enemyId][0];
				var parent = map.getTile(
					newX + firstBlock.x,
					newY + firstBlock.y
				).block.parentCharacter;

				
				var theirSize = parent.blocks.length;

				if(ourSize >= theirSize){	//absorption, todo: apply rules
					//todo: check to see if player is deleted
					if(parent.equals(pc)){
						game.state="dead";
					}

					var blockXy = [];

					for(var i=0;i<parent.blocks.length;i++){
						var pb = parent.blocks[i];
						blockXy.push({
							x:(parent.x+pb.x)-this.x,
							y:(parent.y+pb.y)-this.y,
							item: pb.item
						});

						//needed if reassigning item
						pb.item = null;

						/*blockXy.push({
							x:parent.x-this.x+pb.x,
							y:parent.y-this.y+pb.y
						});*/
					}


					game.deleteMonster(parent);

					for(var i=0;i<blockXy.length;i++){
						var xy = blockXy[i];
						var newBlock = new Block(
							xy.x,
							xy.y,
							this
						);
						
						if(xy.item!=null){
							newBlock.item = xy.item;
							xy.item.parentBlock = newBlock;
						}

						this.blocks.push(newBlock);

					}



					
				}else{	//destroy blocks
					for(var i=0;i<splitBlocks[enemyId].length;i++){
						var oldBlock = map.getTile(
							newX + splitBlocks[enemyId][i].x,
							newY + splitBlocks[enemyId][i].y
						).block;

						if(oldBlock.isCenter()){
							if(parent.equals(pc)){
								game.state="dead";
							}

							game.deleteMonster(parent);
							parent = null;
							break;
						}

						oldBlock.deleteYoself();
					}
					if(parent != null){
						parent.deleteOrphanBlocks();
					}
				}
			}

			this.animationOffset.x += dx * map.tileSize*.333;
			this.animationOffset.y += dy * map.tileSize*.333;

		//free movement
		}else{
			this.moveToTile(newTile);

			this.animationOffset.x += -dx * map.tileSize;
			this.animationOffset.y += -dy * map.tileSize;
		}



		return true;

		/*this.moveToTile(newTile);

		this.animationOffset.x += -dx * map.tileSize;
		this.animationOffset.y += -dy * map.tileSize;*/


		
	}
}