game = {
	version: 0.9,
	viewRadius: 9.5,
	zoom:1,
	targetZoom:1,
	canvasWidth:null,
	canvasHeight:null,
	monsters: [],
	patterns: [],
	//the idea here is we want enemy animations to occur slowly so you can see wtf is happening
	queuedMoves:[],
	tickQueued: false,
	moveInterval:null,
	state: "title",
	deadTimerSet:false,
	tickCount:0,
	cancerCounter: .01,
	score:0,
	itemColors:[
		new Color(0,100,50),//red
		new Color(100,100,50),//green
		new Color(225,100,65),//blue
		new Color(270,100,50),//purple
		new Color(30,100,55),//orange
		new Color(50,100,60),//yellow
		new Color(180,100,60)//cyan
	],
	discoveredItems:[false,false,false,false,false,false],
	crownTile: null,
	gateTile: null,
	level: 1,
	prayer:0,
	prayerDirection: null,
	patternBlockHit: false,
	numTileCanvases:20,
	tileCanvases:[],
	zoomIndex:1,
	zoomLevels:[.3,1,3],
	username: null,
	key: null,
	checksum:"",
	wheelZooming: false,
	speedCounter:0,
	zoomIn:function(){
		game.zoomIndex--;
		if(game.zoomIndex<0){
			game.zoomIndex=0;
		}
		game.targetZoom = game.zoomLevels[game.zoomIndex];

		pc.calculateCanvas();
		for(var i=0;i<game.monsters.length;i++){
			game.monsters[i].calculateCanvas();
		}
	},
	zoomOut:function(){
		game.zoomIndex++;
		if(game.zoomIndex>game.zoomLevels.length-1){
			game.zoomIndex=game.zoomLevels.length-1;
		}
		game.targetZoom = game.zoomLevels[game.zoomIndex];

		pc.calculateCanvas();
		for(var i=0;i<game.monsters.length;i++){
			game.monsters[i].calculateCanvas();
		}
	},
	overlay:function(overlayId, duration,callback){
		if(duration==undefined){
			duration = 700;
		}

		$(".overlay-inner").hide();

		if(overlayId != undefined){
			$("#"+overlayId).fadeIn(duration);
		}

		$("#overlay").hide();
		$("#overlay").fadeIn(duration,callback);
	},
	title:function(){
		var username = game.getUsername();
		$("#username").html(username);
		game.updateScores();
		$(".menu-item").css({opacity:0});
		$("#username-container").css({opacity:0});
		$("#scores").css({opacity:0});
		game.overlay("title",1200,function(){
			$(".menu-item").animate({opacity:1});
			$("#scores").animate({opacity:1});
			$("#username-container").animate({opacity:1});
		});
		game.state="title";
	},
	completeLevel:function(){
		game.state="nextLevel";
		game.level++;
		game.effectiveLevel = game.level;
		if(game.effectiveLevel>7){
			game.effectiveLevel = util.randRange(3,7);
		}
		game.overlay("level-"+game.level,700,function(){
			game.destroyWorlds();
			game.nextLevel();
			setTimeout(function(){
				$("#overlay").fadeOut(500);
			},700);
		});
	},
	nextLevel:function(){
		game.state="nextLevelGeneration";
		clearInterval(game.moveInterval);
		game.deadTimerSet = false;
		game.tickQueued= false;
		game.queuedMoves = [];


		map.generate();

		game.placePlayer();

		//need LOS before we can buffer
		game.computeLos();

		pc.calculateCanvas();

		map.generatePatterns();
		map.generateMonsters();

		game.state="running";
	},
	pushMove:function(move){
		if(game.state=="dead"){
			return;
		}

		if(game.state=="nextLevel" || game.state=="nextLevelGeneration"){
			return;
		}

		game.queuedMoves.push(move);

		while(game.state=="running" && (game.tickQueued || game.queuedMoves.length>1)){
			game.moveSlashTick();
		}
		clearInterval(game.moveInterval);
		game.moveInterval = setInterval(game.moveSlashTick,200);
		game.moveSlashTick();
	},
	moveSlashTick:function(){
		if(game.tickQueued){
			game.tick();
			if(game.state=="running"){
				game.tickQueued = false;
			}else{
				game.queuedMoves = [];
			}
		}else{
			if(game.queuedMoves.length>0){	
				if(game.state!="running"){
					game.queuedMoves = [];
					return;
				}
				var move = game.queuedMoves[0];
				if(pc.tryMove(move.x,move.y)){
					pc.calculateCanvas();
					game.tickQueued = true;
				}
				game.queuedMoves.shift();
			}
		}
	},
	tick: function(){
		game.tickCount++;
		game.speedCounter=0;
		if(game.state=="running" || game.state=="dead"){
			for(var i=0;i<game.monsters.length;i++){
				if(pc.speedPlusPlus <=0 || game.tickCount%2==0){
	  				game.monsters[i].performActions();
	  			}
	  			game.monsters[i].calculateCanvas();
	  			game.monsters[i].invisibility--;
	  			game.monsters[i].speedPlusPlus--;
	  		}
		}
		
  		//if(game.state=="running"){
  			pc.calculateCanvas();
  		//}

  		pc.invisibility--;
	  	pc.speedPlusPlus--;

		if(game.state=="dead" && !game.deadTimerSet){
			game.addScore(game.level, game.score, false);

			game.deadTimerSet = true;
			setTimeout(function(){
				game.destroyWorlds();
				//game.start();
				game.title();
			},1600);
		}
	},
	destroyWorlds:function(){
		for(var i=0;i<game.monsters.length;i++){
			game.deleteMonster(game.monsters[i]);
		}
	},
	end:function(){
		game.frameEnding = frameCounter;
		game.addScore(game.level, game.score, true);
		game.targetZoom = 1;
		game.zoom = 1;

		pc.x=Math.floor(map.mapSize/2);
		pc.y=Math.floor(map.mapSize/2);
		game.cancerChance = .01;
		game.destroyWorlds();
		for(var i=0;i<map.mapSize;i++){
			for(var j=0;j<map.mapSize;j++){
				var tile = new Ground(i,j);
				tile.explored = true;
				map.tiles[i][j] = tile;
			}
		}
		game.state="end";
	},
	start: function(){
		$("#outer").css({opacity:0});
		game.overlay("level-1",700,function(){
			$("#outer").animate({opacity:1});
			setTimeout(function(){
				$("#overlay").fadeOut(500);
			},700);
		});
		game.level = 1;
		game.effectiveLevel = 1;

		game.targetZoom = 1;
		game.zoom = 1;
		game.zoomIndex = 1;

		game.generateTileCanvases();
		game.score = 0;
		game.prayer = 0;

		game.prayerDanger = .05;

		game.prayerDirection = null;
		game.tickCount = 0;
		clearInterval(game.moveInterval);
		game.state="running";
		game.deadTimerSet = false;
		game.tickQueued= false;
		game.queuedMoves = [];

		util.shuffle(game.itemColors);
		game.discoveredItems =[false,false,false,false,false,false,false];

		game.checksum = "";

		map.generate();


		pc = new Character([
			{x:0,y:0}/*,
			{x:1,y:0},
			{x:1,y:1},
			{x:0,y:1},
			{x:2,y:0},
			{x:2,y:1},
			{x:2,y:2},
			{x:3,y:0},
			{x:3,y:1},
			{x:3,y:2},
			{x:-1,y:0},
			{x:-1,y:1},
			{x:-1,y:2},
			{x:-2,y:0},
			{x:-2,y:1},
			{x:-2,y:2},
			{x:-3,y:0},
			{x:-3,y:1},
			{x:-3,y:2},
			{x:-3,y:3},
			{x:-3,y:4}*/
		],15,100,4);

		//pc.grow(pc.blocks[0],.9,.9)

		game.placePlayer();

		
		//need LOS before we can buffer
		game.computeLos();

		pc.calculateCanvas();



		map.generatePatterns();
		map.generateMonsters();


		game.crownPieces = [];
		for(var i=0;i<20;i++){
			game.crownPieces.push({
				speed: util.randRange(10,150),
				radius: Math.random()*.1+.025*i+.3,
				color: new Color(util.randRange(35,45),100,util.randRange(0,75)),
				size: util.randRange(1,2)
			})
		}
		for(var i=0;i<200;i++){
			game.calculateCrown(i);
		}

		game.gatePieces = [];
		for(var i=0;i<20;i++){
			game.gatePieces.push({
				speed: util.randRange(10,60),
				radius: Math.random()*.1+.025*i+.4,
				color: new Color(
					util.randRange(230,270),
					100,
					Math.max(0,120-i*1.5*util.randRange(0,5)) % 100
				),
				size: util.randRange(1,2)
			})
		}

		for(var i=0;i<200;i++){
			game.calculateGate(i);
		}
	},
	placePlayer:function(){
		var timeout = 0;
		while(!pc.place()){
			game.destroyWorlds();
			map.generate();
			timeout++;
			if(timeout>10000){
				alert("sorry, placing player failed. everything is broken")
				break;
			}
		}
	},
	drawPatterns:function(ctx){
		for(var i=0;i<game.patterns.length;i++){
  			if(game.patterns[i].visible || (game.targetZoom<1 && game.patterns[i].discovered)){
  				game.patterns[i].draw(ctx);
  			}
  		}
	},
	handleMonsters:function(ctx){
		for(var i=0;i<game.monsters.length;i++){
  			game.monsters[i].update();
  			if(game.monsters[i].visible){
  				game.monsters[i].draw(ctx);
  			}
  		}
	},
	deleteMonster:function(monster){
		for(var i=0;i<game.monsters.length;i++){
			if(game.monsters[i].equals(monster)){
				for(var j=monster.blocks.length-1;j>=0;j--){
					monster.blocks[j].deleteYoself();
				}
				game.monsters.splice(i,1);
				return;
			}
		}
	},
	deletePattern:function(pattern){
		for(var i=0;i<game.patterns.length;i++){
			if(game.patterns[i].equals(pattern)){
				for(var j=pattern.blocks.length-1;j>=0;j--){
					pattern.blocks[j].deleteYoself();
				}
				game.patterns.splice(i,1);
				return;
			}
		}
	},
	getCamera: function(){
		return {
			zoom: game.zoom,
			xOffset: Math.floor(-pc.x + game.canvasWidth/map.tileSize/2.0/game.zoom)*map.tileSize*game.zoom - pc.animationOffset.x*game.zoom,
			yOffset: Math.floor(-pc.y + game.canvasHeight/map.tileSize/2.0/game.zoom)*map.tileSize*game.zoom - pc.animationOffset.y*game.zoom
		}
	},
	losTiles:{},
	computeLos:function(){
		//mark all monsters as invisible
		for(var i=0;i<game.monsters.length;i++){
			game.monsters[i].visible = false;
		}
		for(var i=0;i<game.patterns.length;i++){
			game.patterns[i].visible = false;
		}

		game.losTiles = {};
		var lightPasses = function(x, y){
		    if(map.getTile(x,y).blocking){
		    	return false;
		    }else{
		    	return true;
		    }
		}
		var fov = new ROT.FOV.PreciseShadowcasting(lightPasses);
		fov.compute(pc.x,pc.y, game.viewRadius, function(x, y, r, visibility) {
			var tile = map.getTile(x,y);
			var block = tile.block;
			var patternBlock = tile.patternBlock;
			if(visibility>0 && util.distance(pc.x,pc.y,x,y)<game.viewRadius+1){
				game.losTiles[x+','+y] = true;
				tile.explored = true;
				tile.visible = true;
				if(block!=null){
					block.parentCharacter.visible = true;
				}
				if(patternBlock!=null){
					patternBlock.parentPattern.visible = true;
					patternBlock.parentPattern.discovered = true;
				}
			}else{
				tile.visible = false;
			}
		});
	},
	los:function(x,y){
		if(game.losTiles[x+','+y]){
			return true;
		}else{
			return false;
		}
	},
	increaseScore:function(val, scoreSize){
		var id = "score-"+util.randRange(1000000,9999999);
		game.score+=val;
		$("#outer").append('<div id="'+id+'" class="plus-score">+'+val+'</div>');
		$("#"+id).animate(
			{
				bottom: '200px',
				opacity: 0
			},
			1500,
			'swing',
			function(){
				$("#"+id).remove();
			}
		);
		game.checksum += val+","+scoreSize+","+game.level+","+pc.x+","+pc.y+","+game.score+" ";
	},
	updateScores:function(){
		var scores = game.getScores();
		var scoreHtml = "";

		if(scores.length>0){
			var newest = scores.concat().sort(function(a,b){
				return new Date(b.actualDate)-new Date(a.actualDate);
			})[0].actualDate;

			scoreHtml+='<table>';
			scoreHtml+='<tr><th>Level</th><th>Score</th><th>Fate</th></tr>';
			for(var i=0;i<Math.min(10,scores.length);i++){
				if(scores[i].won){
					var wonMessage = 'ESCAPED';
				}else{
					var wonMessage = 'KILLED';
				}


				scoreHtml+='<tr class="';
				if(scores[i].actualDate==newest){
					scoreHtml+="newest-score";
				}
				scoreHtml+='">'+
					'<td>'+scores[i]['floor']+'</td>'+
					'<td>'+scores[i]['score']+'</td>'+
					'<td>'+wonMessage+'</td>';
			}
			scoreHtml+='</table>'
		}
		
		$("#scores").html(scoreHtml);
	},
	getUsername:function(){
		var usernameStorageName = "dumuzid.username";
		if(game.username == null){
			//check local storage first
			if(localStorage[usernameStorageName]){
				game.username = localStorage[usernameStorageName];
			}else{
				var cities = ['eridu','badtibira','larsa','sippar','shuruppak','uruk','kish','ur','nippur','lagash','girsu','umma','hamazi','adab','mari','akshak','akkad', 'isin'];
				var city = cities[util.randRange(0,cities.length-1)];
				game.username = city + util.randRange(1,9999);
				localStorage[usernameStorageName] = game.username;
			}
		}
		return game.username;
	},
	changeUsername:function(){
		var usernameStorageName = "dumuzid.username";
		var newUsername = prompt("Enter a leaderboard name:");
		if(newUsername==""){
			return;
		}
		game.username = newUsername;
		localStorage[usernameStorageName] = newUsername;
		$("#username").html(newUsername);

		$.post(
				'http://humbit.com/dumuzid_game/change.php',
				{
					u: newUsername,
					k: game.getKey()
				},
				function(){
				}
			);
	},
	getKey:function(){
		var keyStorageName = "dumuzid.key";
		if(game.key == null){
			//check local storage first
			if(localStorage[keyStorageName]){
				game.key = localStorage[keyStorageName];
			}else{
				game.key = Date.now()+""+util.randRange(100000000,999999999);
				localStorage[keyStorageName] = game.key;
			}
		}
		return game.key;
	},
	addScore: function(floor, score, won){
		var scores = [];
		var storageName = "dumuzid."+game.version+".scores";
		var keyStorageName = "dumuzid.key";
		if(game.localStoragedSupported()){
			if(localStorage[storageName] && localStorage[storageName]!== undefined){
				scores = JSON.parse(localStorage[storageName]);
			}
			
			scores.push({
				floor: floor,
				score: score,
				won: won,
				actualDate: new Date()
			});
			
			localStorage[storageName] = JSON.stringify(scores);

			var username = game.getUsername();
			var key = game.getKey();
			var checksum = game.checksum;

			var wonInt = 0;
			if(won){
				wonInt = 1;
			}

			$.post(
				'http://humbit.com/dumuzid_game/submit.php',
				{
					u: username,
					k: key,
					c: checksum,
					l: floor,
					s: score,
					w: wonInt
				},
				function(){
				}
			);
			//make ajax call to server
			/*	pass in
				u = username (generate one and let user prompt)
				s = score
				l = floor
				w = won
				c = checksum 
				k = key (generate one first time)
				
			*/
		}
	},
	getScores: function(){
		var scores = [];
		var storageName = "dumuzid."+game.version+".scores";

		if(game.localStoragedSupported() && localStorage[storageName]){
			scores = JSON.parse(localStorage[storageName]);

			return scores.sort(function(a,b){
				if(b.score==a.score){
					return b.floor-a.floor;
				}else{
					return b.score-a.score;
				}
			});
		}else{
			return [];
		}
	},
	localStoragedSupported: function() {
		try {
			return 'localStorage' in window && window['localStorage'] !== null;
		} catch (e) {
			return false;
		}
	},
	generateTileCanvases:function(){
		for(var i=0;i<game.numTileCanvases;i++){
			game.tileCanvases.push(
				game.generateTileCanvas(i)
			);
		}
		game.tileCanvases.push(
			game.generateTileCanvas(i, true)
		);
	},
	generateTileCanvas:function(tileIndex, outerWall){
		var canvas = document.createElement('canvas');

	    canvas.width = map.tileSize;
	    canvas.height = map.tileSize;

	    var ctx = canvas.getContext('2d');


		if(outerWall){
			var baseColor = new Color(24,36,40).string;
			var highlightColor = 'black';
			var tilePoints = [];
		}else if(tileIndex<game.numTileCanvases/2.0){
			var baseColor = new Color(230,60,43).string;
			var highlightColor = 'white';
			var tilePoints = [{
				points:[{
					x:-.015,
					y:.8
				},{
					x:1.015,
					y:.8
				},{
					x:1.015,
					y:1.2
				},{
					x:-.015,
					y:1.2
				}],
				color: function(){
					return (new Color(util.randRange(35,40),util.randRange(60,80),util.randRange(70,80))).string;
				}
			}];
		}else{
			var baseColor = new Color(230,60,43).string;
			var highlightColor = 'white';
			var tilePoints = [{
				points:[{
					x:-.15,
					y:.95
				},{
					x:.25,
					y:.4
				},{
					x:.65,
					y:.95
				}],
				color: function(){
					return (new Color(util.randRange(35,40),util.randRange(60,80),util.randRange(70,80))).string;
				}
			},{
				points:[{
					x:.35,
					y:.95
				},{
					x:.75,
					y:.4
				},{
					x:1.15,
					y:.95
				}],
				color: function(){
					return (new Color(util.randRange(35,40),util.randRange(60,80),util.randRange(70,80))).string;
				}
			},{
				points:[{
					x:-.15,
					y:-.15
				},{
					x:.25,
					y:.4
				},{
					x:.65,
					y:-.15
				}],
				color: function(){
					return (new Color(util.randRange(35,40),util.randRange(60,80),util.randRange(70,80))).string;
				}
			},{
				points:[{
					x:.35,
					y:-.15
				},{
					x:.75,
					y:.4
				},{
					x:1.15,
					y:-.15
				}],
				color: function(){
					return (new Color(util.randRange(35,40),util.randRange(60,80),util.randRange(70,80))).string;
				}
			},{
				points:[{
					x:-.015,
					y:.8
				},{
					x:1.015,
					y:.8
				},{
					x:1.015,
					y:1.2
				},{
					x:-.015,
					y:1.2
				}],
				color: function(){
					return (new Color(util.randRange(35,40),util.randRange(60,80),util.randRange(70,80))).string;
				}
			}];
		}

		var x = 0;
		var y = 0;
		var w = map.tileSize;
		
		ctx.save();

		
			ctx.beginPath();
			ctx.fillStyle = baseColor;
			ctx.rect(
				x,
				y,
				w,
				w
			);
			ctx.fill();
	ctx.clip();
		
		for(var i=0;i<500;i++){
			ctx.save();
			ctx.beginPath();
			ctx.globalAlpha = Math.random()*.3;
			ctx.fillStyle = highlightColor;
			ctx.rect(
				x+w*(Math.random()*2-.5),
				y+w*(Math.random()*2-.5),
				w*Math.random()*.1,
				w*Math.random()*.1
			);
			ctx.fill();
			ctx.restore();
		}
		ctx.globalAlpha = 1;
		
		for(var i=0;i<tilePoints.length;i++){
			//broken tiles and shit
			if(Math.random()<.05){
				continue;
			}
			var tp = tilePoints[i];
			ctx.fillStyle = tp.color();
			ctx.save();
			ctx.beginPath();
			for(var j=0;j<tp.points.length;j++){
				var pointX = x + tp.points[j].x*w + w*(Math.random()*.1-.05);
				var pointY = y + tp.points[j].y*w + w*(Math.random()*.1-.05);
				
				if(j==0){
					ctx.moveTo(pointX,pointY);
				}else{
					ctx.lineTo(pointX,pointY);
				}

				if(j==tp.points.length-1){
					ctx.closePath();
					ctx.fill();
					ctx.restore();
				}
			}
		}
		ctx.restore();
		return canvas;
	},
	howto: function(){
		$("#outer").css({opacity:0});
		game.state ="howto";
		game.overlay("howto-1");
		game.howtoCounter=1;
	},
	howtoIncrement:function(){
		game.howtoCounter++;
		if(game.howtoCounter>4){
			game.title();
		}else{
			game.overlay("howto-"+game.howtoCounter);
		}
	},
	credits:function(){
		$("#outer").css({opacity:0});
		game.state ="credits";
		game.overlay("credits");
	},
	calculateCrown: function(fakeCounter){
		var camera = game.getCamera();

		var numDisplay = game.crownPieces.length;

		if(fakeCounter==undefined){
			fakeCounter = frameCounter;
		}

		for(var i=0;i<numDisplay;i++){
			var cp = game.crownPieces[i];
			var size = cp.size;
			game.crownCtx.fillStyle = cp.color.string;
			game.crownCtx.fillRect(
				Math.floor(((1.5+cp.radius*Math.cos(cp.speed*fakeCounter/560))
					*map.tileSize-cp.size/2.0)),
				Math.floor(((1.5+cp.radius*Math.sin(cp.speed*fakeCounter/560))
					*map.tileSize-cp.size/2.0)),
				Math.ceil(size),
				Math.ceil(size)
			);
		}
	},
	calculateGate: function(fakeCounter){
		var camera = game.getCamera();

		var numDisplay = game.gatePieces.length;

		if(fakeCounter==undefined){
			fakeCounter = frameCounter;
		}
		for(var i=0;i<numDisplay;i++){
			var cp = game.gatePieces[i];
			var size = cp.size;
			game.gateCtx.fillStyle = cp.color.string;
			game.gateCtx.fillRect(
				(1.5+.7*cp.radius*Math.pow(Math.cos(cp.speed*fakeCounter/560),3))
					*map.tileSize-cp.size/2.0,
				(1.5+.7*cp.radius*Math.pow(Math.sin(cp.speed*fakeCounter/560),3))
					*map.tileSize-cp.size/2.0,
				size,
				size
			);
		}
	},
	leaderboards:function(){
		window.open('http://humbit.com/dumuzid_game/','_blank');
	}
}