Item = function(parentTile,parentBlock, itemType){
	this.parentTile = parentTile;
	this.parentBlock = parentBlock;

	if(itemType!=undefined){
		this.itemType = itemType;
		this.color = game.itemColors[itemType];
		this.desat = this.color.compute({os:0});
	}
}

Item.prototype.pickup = function(block,tile){
	if(!block.isCenter() && block.item == null){
		this.parentBlock = block;
		this.parentTile = null;
		block.item = this;
		tile.item = null;
	}
}

Item.prototype.drop = function(){
	if(this.parentBlock != null){
		this.parentTile = this.parentBlock.getTile();
		this.parentTile.item = this;
		this.parentBlock.item = null;
		this.parentBlock = null;
	}
}

Item.prototype.discoverAndDestroy = function(){
	game.discoveredItems[this.itemType] = true;
	this.destroy();
}

Item.prototype.destroy = function(){
	if(this.parentBlock != null){
		this.parentBlock.item = null;
		this.parentBlock = null;
	}
}

Item.prototype.use = function(){
	this.discoverAndDestroy();
}

Item.prototype.draw = function(ctx, visible, x, y, radius){
	var color;

	if(visible){
		color = this.color.string;
	}else{
		color = this.desat.string;
	}

	var colors = [
		['black',1.333],
		[color,1]
	];

	if(!game.discoveredItems[this.itemType]){
		for(var j=0;j<colors.length;j++){
			ctx.fillStyle = colors[j][0];
			ctx.beginPath();
		  	ctx.arc(x,y,radius*colors[j][1],0,Math.PI*2,false);
		  	ctx.fill();
		 }
	}else{
		for(var j=0;j<colors.length;j++){
			ctx.fillStyle = colors[j][0];
			ctx.beginPath();
			ctx.moveTo(
				x+radius*this.atPoints[0][0]*colors[j][1],
				y+radius*this.atPoints[0][1]*colors[j][1]
			);
			for(var i=1;i<this.atPoints.length;i++){
				ctx.lineTo(
					x+radius*this.atPoints[i][0]*colors[j][1],
					y+radius*this.atPoints[i][1]*colors[j][1]
				);
			}
			ctx.closePath();
		    ctx.fill();
		}
	}
}



Decay = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,0);
	this.atPoints = [
		[0,-1],
		[1,0],
		[0,1],
		[-1,0]
	];
}

Decay.prototype = new Item();
Decay.prototype.constructor = Item;

Decay.prototype.use = function(){
	var monster = this.parentBlock.parentCharacter;
	var parentBlock = this.parentBlock;

	this.discoverAndDestroy();

	parentBlock.deleteYoself();
	monster.deleteOrphanBlocks();
}

Warp = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,1);
	this.atPoints = [
		[0,-1],
		[1,.5],
		[-1,.5]
	];
}

Warp.prototype = new Item();
Warp.prototype.constructor = Item;

Warp.prototype.use = function(){
	this.parentBlock.parentCharacter.place(true);
	this.discoverAndDestroy();
}


Reveal = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,2);
	this.atPoints = [
		[-1.25,-.7],
		[1.25,-.7],
		[1.25,.7],
		[-1.25,.7]
	];
}

Reveal.prototype = new Item();
Reveal.prototype.constructor = Item;

Reveal.prototype.use = function(){
	for(var i=0;i<map.mapSize;i++){
		for(var j=0;j<map.mapSize;j++){
			map.tiles[i][j].explored=true;
		}
	}
	for(var i=0;i<game.patterns.length;i++){
		game.patterns[i].discovered = true;
	}
	this.discoverAndDestroy();
}




Blast = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,3);
	this.atPoints = [
		[-.5,-1],
		[.5,-1],
		[1,0],
		[.5,1],
		[-.5,1],
		[-1,0]
	];
}

Blast.prototype = new Item();
Blast.prototype.constructor = Item;

Blast.prototype.use = function(){
	var bx = this.parentBlock.getRealX();
	var by = this.parentBlock.getRealY();

	var r = Math.ceil(game.viewRadius/2);
	for(var x=bx-r+1;x<bx+r;x++){
		for(var y=by-r+1;y<by+r;y++){
			var damagedTile = map.getTile(x,y);
			var dist = util.distance(bx,by,x,y)

			if(dist<r){
				if(damagedTile.blocking){
					var newGround = new Ground(x,y);
					newGround.shake = 40*(game.viewRadius-dist)/game.viewRadius;
					map.setTile(x,y,newGround);
				}else{
					damagedTile.shake = 40*(game.viewRadius-dist)/game.viewRadius;
				}
			}
		}
	}

	this.discoverAndDestroy();
}

Bomb = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,4);
	this.atPoints = [
		[0,-1],
		[.25,-.25],
		[1,-.25],
		[.5,.25],
		[.75,1],
		[0,.5],
		[-.75,1],
		[-.5,.25],
		[-1,-.25],
		[-.25,-.25]
	];
}

Bomb.prototype = new Item();
Bomb.prototype.constructor = Item;

Bomb.prototype.use = function(){
	var bx = this.parentBlock.getRealX();
	var by = this.parentBlock.getRealY();

	var r = Math.floor(game.viewRadius-3);
	for(var x=bx-r+1;x<bx+r;x++){
		for(var y=by-r+1;y<by+r;y++){
			var bombedTile = map.getTile(x,y);
			var damagedBlock = bombedTile.block;
			var dist = util.distance(x,y,bx,by);
			if(dist<r){
				if(bombedTile.blocking){
					bombedTile.shake = 20*(game.viewRadius-dist)/game.viewRadius;
				}else{
					bombedTile.shake = 40*(game.viewRadius-dist)/game.viewRadius;
				}

				if(damagedBlock!=null){
					var parent = damagedBlock.parentCharacter;
					if(!parent.equals(this.parentBlock.parentCharacter)){

						if(damagedBlock.isCenter()){
							if(parent.equals(pc)){
								game.state="dead";
							}
							game.deleteMonster(parent);
							parent=null;
						}

						damagedBlock.deleteYoself();

						if(parent != null){
							parent.deleteOrphanBlocks();
						}
					}
				}
			}
		}
	}

	this.discoverAndDestroy();
}


Shoes = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,5);
	this.atPoints = [
		[0,-1.5],
		[.25,-.25],
		[1.5,0],
		[.25,.25],
		[0,1.5],
		[-.25,.25],
		[-1.5,0],
		[-.25,-.25]
	];
}

Shoes.prototype = new Item();
Shoes.prototype.constructor = Item;

Shoes.prototype.use = function(){
	game.speedCounter=0;
	//this seems really wrong, but need to do it
	//player should START with a free move
	if(game.tickCount%2==1){
		game.tickCount++;
	}

	this.parentBlock.parentCharacter.speedPlusPlus = 20;
	this.discoverAndDestroy();
}


Tumor = function(parentTile,parentBlock){
	Item.call(this,parentTile,parentBlock,6);
	this.atPoints = [
		[-1,-1],
		[1,-1],
		[1,1],
		[-1,1]
	];
}

Tumor.prototype = new Item();
Tumor.prototype.constructor = Item;

Tumor.prototype.use = function(dx,dy){
	var parent = this.parentBlock.parentCharacter;
	var tile = this.parentBlock.getTile();
	var x = tile.x+dx;
	var y = tile.y+dy;

	if(x>0 && x<map.mapSize-1 && y>0 && y<map.mapSize-1){
		var newGround = new Ground(x,y);
		map.setTile(x,y,newGround);

		var newBlock = new Block(
			x - parent.x,
			y - parent.y,
			parent
		);
		parent.blocks.push(newBlock);
	}

	this.discoverAndDestroy();
}