map={
	mapSize:60,
	tiles:[],
	tileSize:30,
	lineSize:1,
	generatePatterns:function(){
		game.patterns = [];
		var numPatterns = 5;
		var timeout=0;
		for(var i=0;i<numPatterns;i++){
			timeout++;
			if(timeout>1000){
				break;
			}
			var pattern = new Pattern();

			game.patterns.push(pattern);

			if(pattern.place(true)){
				pattern.calculateCanvas();
			}else{
				game.deletePattern(pattern);
				i-1;
			}
		}
	},
	generateMonsters:function(){
		game.monsters = [];
		var effectiveLevel = Math.min(game.level, 7);
		var numMonsters = effectiveLevel*4;
		var timeout=0;
		for(var i=0;i<numMonsters;i++){
			timeout++;
			if(timeout>1000){
				break;
			}
			var monster = new Character([],util.randRange(0,359), 80, 60);

			game.monsters.push(monster);

			if(monster.place(undefined, true)){
				monster.calculateCanvas();
			}else{
				game.deleteMonster(monster);
				i-1;
			}
		}
	},
	generate:function(){
		//ROT.RNG.setSeed(134);
		for(var i=0;i<map.mapSize;i++){
			map.tiles[i]=[];
			for(var j=0;j<map.mapSize;j++){
				map.tiles[i][j] = new Wall(i,j);
			}
		}

		var dugPercentage = .15+game.level*.05;
		if(game.level==7){
			dugPercentage=.9;
		}

		//create rooms
		var dig = new ROT.Map.Digger(map.mapSize-4,map.mapSize-4,{
			dugPercentage: dugPercentage
		});
		dig.create();
		var rooms = dig.getRooms();
		for (var i=0; i<rooms.length; i++) {
		    var room = rooms[i];
		    for(var x=room.getLeft()+2;x<=room.getRight()+2;x++){
		    	for(var y=room.getTop()+2;y<=room.getBottom()+2;y++){
					map.setTile(x,y,new Ground(x,y));
		    	}
		    }

		    room.getDoors(function(x,y){
		    	//map.setTile(x,y,new Ground(x,y));

		   	 	//var radius = util.randRange(0,2);
				map.grow(x+2,y+2,1);
		    });
		}
		var corridors = dig.getCorridors();
		for (var i=0; i<corridors.length; i++) {
		    var corridor = corridors[i];
		    
		    var rand = Math.random();
		    var radius;
		    if(rand<.6){
		    	radius = 1;
		    }else if(rand<.75){
		    	radius = 2;
		    }else if(rand<.85){
		    	radius = 3;
		    }else if(rand<.95){
		    	radius = 0;
		    }else{
		    	radius = 4;
		    }

		    //if(game.level==7){
		    	//radius+=1;
			//}

		    var x1 = Math.min(corridor._startX, corridor._endX);
		    var x2 = Math.max(corridor._startX, corridor._endX);
		    var y1 = Math.min(corridor._startY, corridor._endY);
		    var y2 = Math.max(corridor._startY, corridor._endY);
		    for(var x=x1+2;x<=x2+2;x++){
		    	for(var y=y1+2;y<=y2+2;y++){
		    		var tile = new Ground(x,y);
					map.setTile(x,y,tile);
					if(radius==0){
						tile.smallCorridor = true;
					}else{
						map.grow(x,y,radius);
					}
		    	}
		    }
		}

		//put down some diamonds
		var numDiamonds = util.randRange(2,4);
		for(var i=0;i<numDiamonds;i++){
			var openTile = map.findOpenTile();
			if(openTile!=null){
				map.setTile(
					openTile.x,
					openTile.y,
					new Diamond(openTile.x,openTile.y)
				);
			}
		}

		//CROWN
		var openTile = map.findOpenTile();
		if(openTile!=null){
			game.crownTile = openTile;
		}

		if(game.level >=7){
			var openTile = map.findOpenTile();
			if(openTile!=null){
				game.gateTile = openTile;
			}
		}
	},
	//find a tile with plenty of breathing space
	findOpenTile: function(){
		for(var j=0;j<1000;j++){
			var randomTile = map.randomTile();
			var neighborhood = randomTile.getAdjacentNeighbors();
			neighborhood.push(randomTile);
			var nonblocking = true;
			for(var k=0;k<neighborhood.length;k++){
				if(neighborhood[k].blocking || neighborhood[k].diamond){
					nonblocking=false;
					break;
				}
			}
			if(nonblocking){
				return randomTile;
			}
		}
		return null;
	},
	grow:function(xc,yc,radius){
		//EXTRA CORRIDOR WIDTH
		for(var x=xc-radius;x<=xc+radius;x++){
			for(var y=yc-radius;y<=yc+radius;y++){
				if(x>1 && x<map.mapSize-2 && y>1 && y<map.mapSize-2){
					map.setTile(x,y,new Ground(x,y));
				}
			}
		}
		
	},
	inbounds:function(x,y){
		if(x>=0 && x<map.mapSize && y>=0 && y<map.mapSize){
			return true
		}else{
			return false;
		}
	},
	getTile:function(x,y){
		if(!map.inbounds(x,y)){
			return new Ground(x,y);
		}else{
			return map.tiles[x][y];
		}
	},
	setTile:function(x,y,tile){
		//if not within walls, just ignore
		if(!(x<1 || x>=map.mapSize-1 || y<1 || y>=map.mapSize-1)){
			map.tiles[x][y] = tile;
		}
	},
	randomTile:function(){
		return map.getTile(
			util.randRange(0,map.mapSize-1),
			util.randRange(0,map.mapSize-1)
		);
	},
	draw:function(ctx){
		var halfTiles = Math.ceil(game.canvasWidth/map.tileSize/2.0/game.zoom)+1;
		var left = Math.max(0,pc.x - halfTiles);
		var right = Math.min(map.mapSize-1,pc.x + halfTiles);
		var top = Math.max(0,pc.y - halfTiles);
		var bottom = Math.min(map.mapSize-1,pc.y + halfTiles);

		
		var camera = game.getCamera();

		var leftX = (left*map.tileSize)*camera.zoom+camera.xOffset;
		var rightX = (right*map.tileSize)*camera.zoom+camera.xOffset; 
		var bottomY = (bottom*map.tileSize)*camera.zoom+camera.yOffset;
		var topY = (top*map.tileSize)*camera.zoom+camera.yOffset; 

		//we fill in this dark blue color, so it is below shaking tiles AND so half of the unseen tiles can use it
		ctx.fillStyle = 'hsl(240,10%,60%)';
		ctx.fillRect(
			leftX,
			topY,
			rightX-leftX,
			bottomY-topY
		);

		//todo: limit by visible tiles
		for(var i=left;i<=right;i++){
			for(var j=top;j<=bottom;j++){
				if(map.getTile(i,j).shake>2){
					map.getTile(i,j).shake*=.9;
				}else{
					map.getTile(i,j).shake=0;
				}
				map.getTile(i,j).draw(ctx);
			}
		}
	}
}