PatternBlock = function(x,y,parentPattern){
	this.x = x;
	this.y = y;
	this.parentPattern = parentPattern;
	this.visited = false;
	this.visible = false;
	this.discovered = true;

	map.getTile(
		this.getRealX(),
		this.getRealY()
	).patternBlock = this;

}

PatternBlock.prototype.deleteYoself = function(){
	this.getTile().patternBlock = null;

	var parentBlocks = this.parentPattern.blocks;
	for(var i=0;i<parentBlocks.length;i++){
		if(parentBlocks[i].equals(this)){
			parentBlocks.splice(i,1);
			return;
		}
	}
}

PatternBlock.prototype.getRealX = function(){
	return this.parentPattern.x + this.x;
}

PatternBlock.prototype.getRealY = function(){
	return this.parentPattern.y + this.y;
}

PatternBlock.prototype.getTile = function(){
	return map.getTile(this.getRealX(),this.getRealY());
}

PatternBlock.prototype.equals = function(block){
	if(this.x==block.x && this.y==block.y){
		return true;
	}else{
		return false;
	}
}

Pattern = function(){
	this.x=util.randRange(1000,999999999);
	this.y=util.randRange(1000,999999999);

	var color = new Color(util.randRange(0,359),100,util.randRange(40,60));
	this.color1 = color.string;
 	this.color2 = color.compute({dh:180}).string;

	this.color1 = 'white'
 	this.color2 = 'red'

	this.blocks = [
		new PatternBlock(0,0, this)
	];


	var rand = Math.random();
	if(rand<.20){
		var growRate = .5;
		var decayRate = 0;
	}else if(rand<.40){
		var growRate = 0;
		var decayRate = 0;
	}else{
		var growRate = .1*game.effectiveLevel;
		var decayRate = .05*game.effectiveLevel;
	}

	this.grow(this.blocks[0],growRate,decayRate);

	this.score = Math.pow(this.blocks.length,2);
	this.scoreSize = this.blocks.length;

	this.canvases = [];
}

Pattern.prototype.checkMatch = function(pattern){
	if(this.blocks.length == pc.blocks.length){
		var match = true;
		for(var i=0;i<this.blocks.length;i++){
			if(this.blocks[i].getTile().block == undefined ||
			!this.blocks[i].getTile().block.parentCharacter.equals(pc)){
				//match = false;
				return;
			}
		}
		if(match){
			game.increaseScore(this.score, this.scoreSize);
			game.deletePattern(this);
		}
	}
}

Pattern.prototype.equals = function(pattern){
	if(this.x==pattern.x && this.y==pattern.y){
		return true;
	}else{
		return false;
	}
}

Pattern.prototype.grow = function(block,growChance, decay){
	if(this.blocks.length>300){
		return;
	}

	if(!block.visited){
		block.visited = true;
		var directions = util.getDirections();
		for(var i=0;i<directions.length;i++){
			if(Math.random()<=growChance){
				var neighbor = this.findBlock(
					block.x+directions[i].x,
					block.y+directions[i].y
				);
				if(neighbor==null){
					var newBlock = new PatternBlock(
						block.x+directions[i].x,
						block.y+directions[i].y,
						this
					);
					this.blocks.push(newBlock);

					this.grow(newBlock, growChance*decay, decay);
				}
			}
		}
	}
}

Pattern.prototype.findBlock = function(x,y){
	for(var i=0;i<this.blocks.length;i++){
		if(this.blocks[i].x==x && this.blocks[i].y==y){
			return this.blocks[i];
		}
	}
	return null;
}

Pattern.prototype.moveToTile = function(tile){
	for(var i=0;i<this.blocks.length;i++){
		var oldTile = map.getTile(
			this.blocks[i].x+this.x,
			this.blocks[i].y+this.y
		);
		oldTile.patternBlock = null;
	}
	for(var i=0;i<this.blocks.length;i++){
		var newTile = map.getTile(
			this.blocks[i].x+tile.x,
			this.blocks[i].y+tile.y
		);

		newTile.patternBlock = this.blocks[i];
	}

	this.x = tile.x;
	this.y = tile.y;
	this.tile = tile;
}

Pattern.prototype.place = function(extraSpace){
	for(var i=0;i<1000;i++){
		//check to see if we fit
		//todo: more complicated fit logic
		var tile = map.randomTile();
		if(this.fits(tile.x, tile.y) &&
		(extraSpace==undefined ||
		(this.fits(tile.x+1,tile.y)
		&& this.fits(tile.x-1,tile.y)
		&& this.fits(tile.x,tile.y+1)
		&& this.fits(tile.x,tile.y-1)))){
			this.moveToTile(tile);
			return true;
		}
	}
	//console.log('failed to place')
	return false;
}

Pattern.prototype.fits = function(x,y){
	for(var i=0;i<this.blocks.length;i++){
		var tile = map.getTile(
			x+this.blocks[i].x,
			y+this.blocks[i].y
		);
		if(tile.blocking){
			return false;
		}

		if(tile.patternBlock &&
		!this.equals(tile.patternBlock.parentPattern)){
			return false;
		}

		if(tile.diamond){
			return false;
		}
	}
	return true;
}


//saves a canvas for drawing and also calculates corners
Pattern.prototype.calculateCanvas = function(){
	var tileSize = map.tileSize;

	//calculate total size
	//calculate left corner offset

	this.minX=10000;
	this.minY=10000;
	this.maxX=-10000;
	this.maxY=-10000;
	for(var i=0;i<this.blocks.length;i++){
		this.minX = Math.min(this.blocks[i].x, this.minX);
		this.maxX = Math.max(this.blocks[i].x, this.maxX);
		this.minY = Math.min(this.blocks[i].y, this.minY);
		this.maxY = Math.max(this.blocks[i].y, this.maxY);
	}

	var width = this.maxX-this.minX+1;
	var height = this.maxY-this.minY+1;


	for(var c=0;c<3;c++){
		var canvas = document.createElement('canvas');
		this.canvases[c] = canvas;

		canvas.width = tileSize * width+2*map.lineSize;
		canvas.height = tileSize * height+2*map.lineSize;

		var ctx = canvas.getContext('2d');


		ctx.beginPath();
		ctx.fillStyle = this.color1;
		for(var i=0;i<this.blocks.length;i++){
			/*if(map.getTile(
				this.blocks[i].x+this.x,
				this.blocks[i].y+this.y
			).explored || true){*/
				ctx.rect(
					(this.blocks[i].x-this.minX)*tileSize,
					(this.blocks[i].y-this.minY)*tileSize,
					tileSize,
					tileSize
				);
				ctx.fill();
			//}
		}

		ctx.clip();
		//todo: draw 3 separate canvas and alternate based on framerate

		 	ctx.strokeStyle = this.color2;
		 	var numRings = 6;
		 for(var i=0;i<numRings;i++){
				var arcX = canvas.width/2+1*map.tileSize*Math.cos(Math.PI*2*i/numRings);
				var arcY = canvas.height/2+1*map.tileSize*Math.sin(Math.PI*2*i/numRings);
			for(var j=0;j<60;j++){
		    	ctx.beginPath();
		  		ctx.arc(arcX,arcY,
		  		j*9+3*(c),0, Math.PI*2,true);

		  		ctx.stroke();
			}
		}
		/*ctx.strokeStyle = 'white';
		for(var i=0;i<this.blocks.length;i++){
			ctx.rect(
				(this.blocks[i].x-this.minX)*tileSize,
				(this.blocks[i].y-this.minY)*tileSize,
				tileSize,
				tileSize
			);
			ctx.stroke();
		}*/
	}
}

Pattern.prototype.draw = function(canvasCtx){
	if(this.canvases.length==0){
		return;
	}

	var animationSpeed = 20;
	if(game.patternBlockHit){
		animationSpeed = 5;
	}
	var c = ((Math.floor(frameCounter/animationSpeed))%3);
	canvas = this.canvases[c];

	//this.calculateCanvas();

	var camera = game.getCamera();
	var startX = ((this.minX+this.x)*map.tileSize)*camera.zoom + camera.xOffset - map.lineSize;
	var startY = ((this.minY+this.y)*map.tileSize)*camera.zoom + camera.yOffset - map.lineSize;

	canvasCtx.drawImage(canvas, startX, startY, canvas.width*camera.zoom, canvas.height*camera.zoom);

}