function Tile(x,y){
	this.x=x
	this.y=y;

	this.explored = false;
	this.visible = false;

	this.block = null;
	this.patternBlock = null;

	this.item = null;

	this.crown = false;

	this.decayEffect = null;

	this.shake = 0;

	if(this.x==0 || this.y==0 || this.x==map.mapSize-1 || this.y==map.mapSize-1){
		this.tileIndex = game.numTileCanvases;
	}else if(this.y%2==0){
		this.tileIndex = util.randRange(0,game.numTileCanvases/2.0-1);
	}else{
		this.tileIndex = util.randRange(game.numTileCanvases/2.0,game.numTileCanvases-1);
	}
}

Tile.prototype.equals = function(tile){
	if(this.x==tile.x && this.y==tile.y){
		return true;
	}else{
		return false;
	}
}

Tile.prototype.getAdjacentNeighbors = function(shuffle){
	var tiles = [
		map.getTile(this.x+1,this.y),
		map.getTile(this.x-1,this.y),
		map.getTile(this.x,this.y+1),
		map.getTile(this.x,this.y-1)
	];
	if(shuffle){
		return util.shuffle(tiles);
	}else{
		return tiles;
	}
}



function Ground(x,y){
	Tile.call(this, x,y);
	this.blocking = false;

	if((x+y)%2==0){
		this.color = new Color(
			util.randRange(200,220),
			util.randRange(5,15),
			util.randRange(88,92)
		);
	}else{
		this.color = new Color(
			util.randRange(200,220),
			util.randRange(5,15),
			util.randRange(78,82)
		);
	}
	this.desat = this.color.compute({dl:-20,os:0});
	/*this.color = 'hsl('+
		util.randRange(15,20)+','+
		util.randRange(35,40)+'%,'+
		util.randRange(60,65)+'%'+
	')';*/

	var rand = Math.random();
	if(map.inbounds(x,y) && Math.random()<.00666){
		if(rand<.35){
			this.item = new Decay(this,null);
		}else if(rand<.55){
			this.item = new Tumor(this,null);
		}else if(rand<.65){
			this.item = new Blast(this,null);
		}else if(rand<.75){
			this.item = new Shoes(this,null);
		}else if(rand<.85){
			this.item = new Bomb(this,null);
		}else if(rand<.95){
			this.item = new Warp(this,null);
		}else{
			this.item = new Reveal(this,null);
		}
	}
}

Ground.prototype = new Tile();
Ground.prototype.constructor = Tile;


Ground.prototype.draw = function(ctx){
	var camera = game.getCamera();
	var visible = false;

	//skip drawing on half the tiles for performance xD
	var doDraw = true;

	if(!this.explored){
		ctx.fillStyle = 'white';
	}else if(!game.los(this.x,this.y)){
		ctx.fillStyle = this.desat.string;
		if((this.x+this.y)%2==1){
			doDraw = false;
		}
	}else{
		ctx.fillStyle = this.color.string;
		visible = true;
	}

	if(doDraw){
		ctx.fillRect(
			Math.floor(((this.x)*map.tileSize)*camera.zoom+camera.xOffset
				+this.shake*camera.zoom*(Math.random()-.5)
			),
			Math.floor(((this.y)*map.tileSize)*camera.zoom+camera.yOffset
				+this.shake*camera.zoom*(Math.random()-.5)
			),
			Math.ceil(map.tileSize*camera.zoom),
			Math.ceil(map.tileSize*camera.zoom)
		);
	}

	if(this.item!=null && this.explored && game.targetZoom>=1){
		this.item.draw(
			ctx,
			visible,
			Math.floor(((this.x+.5)*map.tileSize)*camera.zoom+camera.xOffset),
	  		Math.floor(((this.y+.5)*map.tileSize)*camera.zoom+camera.yOffset),
	  		Math.ceil(map.tileSize*.2*camera.zoom)
		);
	}

	if(this.decayEffect != null){
		if(this.decayEffect.frameStart < frameCounter-45){
			this.decayEffect = null;
		}else{
			var effectWidth = map.tileSize*camera.zoom*Math.pow(.9,(frameCounter-this.decayEffect.frameStart));
			ctx.fillStyle = this.decayEffect.color;
			ctx.fillRect(
				Math.floor(((this.x+.5)*map.tileSize)*camera.zoom+camera.xOffset)-effectWidth/2.0,
				Math.floor(((this.y+.5)*map.tileSize)*camera.zoom+camera.yOffset)-effectWidth/2.0,
				effectWidth,
				effectWidth
			);
		}
	}
}




function Wall(x,y){
	Tile.call(this, x,y);
	this.blocking = true;

	if(this.x==0 || this.y==0 || this.x==map.mapSize-1 || this.y==map.mapSize-1){
		this.color = new Color(24,36,40);
	}else{
		this.color = new Color(230,60,43);
	}
	this.desat = this.color.compute({os:0});
}

Wall.prototype = new Tile();
Wall.prototype.constructor = Tile;


function Diamond(x,y){
	Tile.call(this, x,y);
	this.blocking = false;
	this.diamond = true;

	this.color = new Color(0,0,0);
	this.desat = this.color.compute({os:0});

	this.ground = new Ground(x,y);
	
	this.points = [
		[0,-1],
		[1,0],
		[0,1],
		[-1,0]
	];
}

Diamond.prototype = new Tile();
Diamond.prototype.constructor = Tile;

Diamond.prototype.draw = function(ctx){
	var camera = game.getCamera();
	var visible = false;

	if(this.explored){
		this.ground.explored = true;
	}

	this.ground.draw(ctx);

	if(!this.explored){
		ctx.fillStyle = 'white';
		return;
	}else if(!game.los(this.x,this.y)){
		ctx.fillStyle = this.desat.string;
	}else{
		ctx.fillStyle = this.desat.string;
		visible = true;
	}


	var x = Math.floor(((this.x+.5)*map.tileSize)*camera.zoom+camera.xOffset);
	var y = Math.floor(((this.y+.5)*map.tileSize)*camera.zoom+camera.yOffset);
	var r = Math.ceil(map.tileSize*.5*camera.zoom);

	ctx.beginPath();
	ctx.moveTo(
		x+r*this.points[0][0],
		y+r*this.points[0][1]
	);
	for(var i=1;i<this.points.length;i++){
		ctx.lineTo(
			x+r*this.points[i][0],
			y+r*this.points[i][1]
		);
	}
	ctx.closePath();
    ctx.fill();
}




Wall.prototype.draw = function(ctx){
	var camera = game.getCamera();
	var visible = false;

	if(!this.explored){
		ctx.fillStyle = 'white';
		ctx.fillRect(
			Math.floor(((this.x)*map.tileSize)*camera.zoom+camera.xOffset),
			Math.floor(((this.y)*map.tileSize)*camera.zoom+camera.yOffset),
			Math.ceil(map.tileSize*camera.zoom),
			Math.ceil(map.tileSize*camera.zoom)
		);
	}else{
		if(game.targetZoom>=1){
			var canvas = game.tileCanvases[this.tileIndex];
			ctx.drawImage(
				canvas,
				Math.floor(((this.x)*map.tileSize)*camera.zoom+camera.xOffset
					+this.shake*camera.zoom*(Math.random()-.5)
				),
				Math.floor(((this.y)*map.tileSize)*camera.zoom+camera.yOffset
					+this.shake*camera.zoom*(Math.random()-.5)
				),
				Math.ceil(map.tileSize*camera.zoom),
				Math.ceil(map.tileSize*camera.zoom)
			);
		}else{
			ctx.fillStyle = this.color.string;
			ctx.fillRect(
				Math.floor(((this.x)*map.tileSize)*camera.zoom+camera.xOffset),
				Math.floor(((this.y)*map.tileSize)*camera.zoom+camera.yOffset),
				Math.ceil(map.tileSize*camera.zoom),
				Math.ceil(map.tileSize*camera.zoom)
			);
		}
		
	}
	

	if(this.item!=null && this.explored){
		this.item.draw(
			ctx,
			visible,
			Math.floor(((this.x+.5)*map.tileSize)*camera.zoom+camera.xOffset),
	  		Math.floor(((this.y+.5)*map.tileSize)*camera.zoom+camera.yOffset),
	  		Math.ceil(map.tileSize*.2*camera.zoom)
		);
	}

	/*}else if(!game.los(this.x,this.y)){
		ctx.fillStyle = this.desat.string;
	}else{
		ctx.fillStyle = this.color.string;
		visible = true;
	}*/

}

Wall.prototype.calculateCanvas = function(){

}

Tile.prototype.drawCrown = function(ctx){
	var camera = game.getCamera();
	
	if(this.explored){
		if(game.targetZoom>=1){	
			game.calculateCrown();
		}
		var startX = Math.floor(((this.x-1)*map.tileSize)*camera.zoom+camera.xOffset);
		var startY = Math.floor(((this.y-1)*map.tileSize)*camera.zoom+camera.yOffset);

		ctx.drawImage(game.crownCanvas,startX,startY, game.crownCanvas.width*camera.zoom, game.crownCanvas.height*camera.zoom);
	}
	

}

Tile.prototype.drawGate = function(ctx){
	if(game.level >=7){
		var camera = game.getCamera();
		
		if(this.explored){
			if(game.targetZoom>=1){	
				game.calculateGate();
			}
			var startX = ((this.x-1)*map.tileSize)*camera.zoom+camera.xOffset;
			var startY = ((this.y-1)*map.tileSize)*camera.zoom+camera.yOffset;

			ctx.drawImage(game.gateCanvas,startX,startY, game.gateCanvas.width*camera.zoom, game.gateCanvas.height*camera.zoom);
		}
	}
}


